# first time screen

import sys
import json
import os.path
from kivy.core.window import Window
from kivy.uix.floatlayout import FloatLayout
from kivy.lang import Builder
from jocular.utils.utils import add_if_not_exists, home_folder
from jocular.widgets.jfilebrowser import JFileBrowser

Builder.load_string('''

#:import joc_icon jocular.utils.utils.joc_icon

<FirstTime>:

    subtitle: _subtitle
    path: _path

    BoxLayout:
        orientation: 'vertical'

        Label:
            size_hint: 1, .3

        JLabel:
            size_hint: 1, .3
            font_size: '64sp'
            markup: True
            text: 'Welcome to J' + joc_icon('oculus', fs='48') + 'cular!'

        JLabel:
            id: _subtitle
            size_hint: 1, None
            text_size: self.size
            halign: 'center'
            height: dp(100)
            font_size: '24sp'
            text: ''
            padding: dp(20), dp(20)

        BoxLayout:
            size_hint: 1, None
            height: dp(50)
            Label:
                size_hint: 1, 1
            Button:
                id: _path
                size_hint: 1, None
                size: self.texture_size
                halign: 'center'
                font_size: '20sp'
                text: ''
                padding: dp(10), dp(10)
                on_press: root.choose_default_path()
            Label:
                size_hint: 1, 1

        BoxLayout:
            size_hint: 1, None
            height: dp(50)
            Label:
                size_hint: 1, 1
            Button:
                size_hint: 1, None
                size: self.texture_size
                halign: 'center'
                font_size: '20sp'
                text: 'Choose a different location ...'
                padding: dp(10), dp(10)
                on_press: root.choose_path()
            Label:
                size_hint: 1, 1

        Label:
            size_hint: 1, .3

        BoxLayout:
            size_hint: 1, None
            height: dp(50)
            Label:
                size_hint: 1, 1
            JButton:
                size_hint: 1, None
                size: self.texture_size
                halign: 'center'
                font_size: '16sp'
                text: 'Exit Jocular'
                padding: dp(10), dp(10)
                on_press: root.exit_jocular()
            Label:
                size_hint: 1, 1

''')


class FirstTime(FloatLayout):

    def __init__(self, scope, **kwargs):
        super().__init__()
        self.scope = scope
        self.subtitle.text = 'Please select a location that Jocular can use to store your captures'
        self.path.text = os.path.join(home_folder(), 'JocularData')
        self.pos = 0, 0
        self.size = Window.size

    def choose_default_path(self):
        self.create_folder(self.path.text)

    def choose_path(self):
        self._browser = JFileBrowser(path=home_folder(),
            title='Choose or create a directory for your Jocular data')
        self.add_widget(self._browser)
        self._browser.bind(
            on_success=self.dirname_chosen,
            on_canceled=self.cancel_init)

    def create_folder(self, user_data_path):
        try:
            add_if_not_exists(user_data_path)
            path = os.path.join('jocular', 'user_path.json')
            with open(path, 'w') as f:
                json.dump({'user_data_path': user_data_path}, f, indent=4)
            self.scope.normal_init(user_data_path, first_time=True, screen=self)
        except Exception:
            self.subtitle.text = 'Problem creating {:} or user_path.json'.format(user_data_path)

    def dirname_chosen(self, instance):
        try:
            path = instance.selection[0]
            self.remove_widget(instance)
            self.create_folder(path)
        except Exception as e:
            self.cancel_init(instance)
            self.subtitle.text = 'Problem: {:}'.format(str(e))

    def cancel_init(self, instance):
        self.remove_widget(instance)

    def exit_jocular(self):
        sys.exit()
