'''Processing. Control panel that appears at the foot of the screen. Mainly a layout class
with support for pinning etc. Could be made more generic in future (with Obj).
'''

from kivy.logger import Logger
from kivy.uix.boxlayout import BoxLayout
from kivy.properties import BooleanProperty, OptionProperty
from kivy.core.window import Window
from kivy.lang import Builder
from kivy.animation import Animation
from kivy.metrics import dp

from jocular.utils.component import Component
from jocular.utils.utils import joc_icon
from jocular.widgets.widgets import ConfigToggle

Builder.load_string('''

#:import joc_icon jocular.utils.utils.joc_icon

<Title@Label>:
    size_hint: 1, None
    height: dp(30)
    text_size: self.size
    valign: 'middle'
    halign: 'center'
    padding: dp(10), dp(5)
    color: app.text_color

<TitlePanel@BoxLayout>:
    size_hint: 1, None
    height: dp(30)

<ControlPanel@BoxLayout>:
    size_hint: 1, None
    height: dp(130)

<Process@BoxLayout>:
    canvas.before:
        Color:
            rgba: app.background_color
        Rectangle:
            pos: self.pos
            size: self.size
    size_hint: 1, None
    orientation: 'vertical'
    scope: root.parent.scope

<CapturePanel@Process>:
    size_hint: None, 1
    width: root.scope.obj.width

    TitlePanel:
        Title:
            text: 'Capture'
            size_hint: 1, 1
        ConfigToggle:
            group: 'config' 
            on_state: root.scope.configurer.open('Capture') 

    ControlPanel:

        Label:
            size_hint: .5, 1

        RelativeLayout:
 
            JSwitch:
                id: _sub_type
                size: dp(130), dp(130)
                presets: {'light': -20, 'dark': 30, 'flat': 60, 'bias':80}
                disabled: not root.scope.live or (_camera_toggle.state == 'down')
                value: root.scope.obj.sub_type

            ToggleButton:
                id: _camera_toggle
                pos_hint: {'center_x': .5, 'center_y': .5}
                size_hint: None, None
                size: dp(24), dp(24)
                text_size: self.size
                halign: 'center'
                valign: 'middle'
                markup: True
                state: 'normal' if root.scope.camera.capture_event is None else 'down'
                text: joc_icon('pause', fs='24') if self.state == 'down' else joc_icon('play', fs='24')
                on_state: root.scope.camera.start(_sub_type.value) if self.state == 'down' else root.scope.camera.stop()
                color: app.text_color if self.state == 'down' else app.lowlight_color
                disabled: not root.scope.live
                background_color: 0, 0, 0, 0
                
        Label:
            size_hint: 1, 1

        JKnob:
            knob_type: 'numeric with presets'
            size: dp(130), dp(130)
            min: 1
            max: 60
            minangle: -140
            maxangle: 140
            curve: 1.8
            line_start: .82
            line_end: .95
            switch_radius: .6
            knob_radius: .54
            value_format: '{:.1f}s'
            value_font_size: '16sp'
            presets: [1, 2, 5, 10, 15, 20, 30, 45, 60]
            value: root.scope.obj.exposure
            on_value: root.scope.obj.exposure = self.value

        Label:
            size_hint: 1, 1

 
<CalibrationPanel@Process>:
    size_hint: None, 1
    width: dp(260)

    TitlePanel:
        Title:
            text: 'Calibrate'
            size_hint: 1, 1

        ConfigToggle:
            group: 'config'
            on_state: root.scope.configurer.open('Calibration') 

    ControlPanel:

        Label:
            size_hint: .05, 1

        JKnob:
            knob_type: 'numeric'
            min: -10
            max: 30
            size: dp(80), dp(80)
            value_type: 'int'
            value_format:       '{:2d}\N{DEGREE SIGN}C'
            value_font_size: '18sp'
            title: 'Temp'
            on_value: root.scope.obj.temperature = self.value
            inactive: root.scope.obj.temperature_inactive
            on_inactive: root.scope.obj.temperature_inactive = self.inactive
            value: root.scope.obj.temperature

        Label:
            size_hint: .05, 1

        BoxLayout:
            size_hint: None, 1
            width: dp(150)
            orientation: 'vertical'

            Label:
                size_hint: 1, .1

            JToggle:
                selected: root.scope.calibrator.apply_BPM
                on_selected: root.scope.calibrator.apply_BPM = self.selected
                label: 'hot pixel removal'

            JToggle:
                selected: root.scope.calibrator.apply_flat
                on_selected: root.scope.calibrator.apply_flat = self.selected
                label: 'apply flats'

            JLabel:
                text: root.scope.calibrator.current_flat[:-3].replace('_',' ')

            JLabel:
                text: root.scope.calibrator.current_bias[:-3].replace('_',' ')

            JToggle:
                selected: root.scope.calibrator.apply_dark
                on_selected: root.scope.calibrator.apply_dark = self.selected
                label: 'apply darks'

            JLabel:
                text: root.scope.calibrator.current_dark[:-3].replace('_',' ')

            Label:
                size_hint: 1, 1

        Label:
            size_hint: .05, 1

<AlignmentPanel@Process>:
    size_hint: None, 1
    width: dp(160)

    TitlePanel:
        Title:
            text: 'Align'
            size_hint: 1, 1
        ConfigToggle:
            group: 'config'
            on_state: root.scope.configurer.open('Alignment') 

    ControlPanel:
        Label:
            size_hint: .05, 1

        BoxLayout:
            size_hint: .9, 1
            orientation: 'vertical'

            Label:
                size_hint: 1, .1

            JToggle:
                size_hint: 1, None
                selected: root.scope.aligner.do_align
                on_selected: root.scope.aligner.do_align = self.selected
                label: 'align'

            Label:
                size_hint: 1, .1

            # not yet implemented fully
            JButton:
                size_hint: 1, None
                height: dp(28)
                # text: joc_icon('realign') + ' realign'
                text: ''
                disabled: True
                on_press: root.scope.stacker.realign_from_current()

            Label:
                size_hint: 1, 1

            JButton:
                size_hint: 1, None
                height: dp(28)
                text: 'recompute {:2.0f}s'.format(root.scope.stacker.update_time)
                disabled: root.scope.stacker.update_time == 0
                on_press: root.scope.stacker.recompute()

            Label:
                size_hint: 1, .5
        Label:
            size_hint: .05, 1

<StackCombinePanel@Process>:
    size_hint: None, 1
    width: dp(130)
    TitlePanel:
        Title:
            text: 'Stack'

    ControlPanel:
        JSwitch:
            size: dp(130), dp(130)
            on_value: root.scope.eyepiece.stack_combine_method = self.value
            value: root.scope.eyepiece.stack_combine_method
            presets: {'mean': 140, 'median': -150, '90%': 50, '80%': 0, '70%': -50}
                
<DisplayPanel@Process>:
    size_hint: None, 1
    width: dp(150)
    TitlePanel:
        Title:
            text: 'Display'
        ConfigToggle:
            group: 'config'
            on_state: root.scope.configurer.open('Display') 

    ControlPanel:
        JSwitch:
            size: dp(150), dp(150)
            value: root.scope.eyepiece.stretch
            on_value: root.scope.eyepiece.stretch = self.value
            presets: {'asinh': 150, 'hyper': 120, 'log': 60, 'linear': 0, 'γ': -90, 'log γ': -150}

<GeneralPanel@Process>:
    size_hint: 1, 1
    TitlePanel:
        Title:
            text: 'General'
            size_hint: 1, 1

        ConfigToggle:
            group: 'config'
            on_state: root.scope.configurer.open('General')
    
    ControlPanel:
        orientation: 'horizontal'

        Label:
            size_hint: .1, 1

        JKnob:
            size_hint: None, None
            knob_type: 'numeric'
            size: dp(80), dp(80)
            min: 0
            max: 100
            value_type: 'int'
            value_format: '{:2d} %'
            title: 'brightness'
            continuous_update: True
            on_value: root.scope.dimmer = self.value
            value: root.scope.dimmer

        Label:
            size_hint: .1, 1

<Processing>:

    canvas.before:
        Color:
            rgba: .05, .05, .05, 1
        Rectangle:
            pos: self.pos
            size: self.size
        Color:
            rgba: app.line_color
        Rectangle:
            pos: self.pos
            size: self.size

    canvas.after:
        Color:
            rgba: app.line_color
        Line:
            points: 0, self.y + self.height, self.width, self.y + self.height
            width: .5
        Line:
            points: 0, self.y + self.height - 30, self.width, self.y + self.height - 30
            width: .5

    orientation: 'horizontal'
    size_hint: None, None
    spacing: 1

    CapturePanel:
    CalibrationPanel:
    AlignmentPanel:
    StackCombinePanel:
    DisplayPanel:
    GeneralPanel:

''')



class Processing(BoxLayout, Component):

    state = OptionProperty('displayed', options=['hidden', 'displayed', 'incoming', 'leaving' ])
    pinned = BooleanProperty(False)

    settables = ['pinned']

    def __init__(self, scope=None, **kwargs):
        self.scope = scope
        super().__init__()
        self.load_settings(settings=scope.settings)
        self.size = Window.width, dp(160)
        self.pos = 0, -self.height

    def on_touch_down(self, touch):
        handled = super().on_touch_down(touch)
        if self.collide_point(*touch.pos):
            return True
        return handled
  
    def pin_pressed(self, inst, value):
        self.pinned = not self.pinned

    def redraw(self, *args):
        Logger.debug('Processing: redraw')
        self.scope.add_widget(self) # , index=self.scope.layout['processing_index'])
        self.size = Window.width, dp(160)
        self.pin = ConfigToggle(text='', size_hint=(None, None), size=(dp(28), dp(28)))
        self.pin.bind(state=self.pin_pressed)
        self.scope.add_widget(self.pin)

        if self.pinned:
            self.pin.pos = self.width - self.pin.width, self.height
            self.pin.text = joc_icon('pinned')
            self.pos = 0, 0
        else:
            self.pos = 0, -self.height
            self.pin.y = self.y
            self.pin.text = joc_icon('unpinned')

    def show(self, *args):
        if self.pinned or (self.state in ['displayed', 'incoming']):
            return
        self.anim = Animation(y=0, duration=.25)
        self.anim.bind(on_start=self.menu_incoming, on_complete=self.menu_displayed)
        self.anim.start(self)

    def hide(self, *args):
        if self.pinned or (self.state in ['leaving', 'hidden']) or self.scope.configurer.showing:
            return
        self.anim = Animation(y=-self.height - 10, duration=.35)
        self.anim.bind(on_start=self.menu_leaving, on_complete=self.menu_hidden)
        self.anim.start(self)

    def on_pinned(self, *args):
        if self.pinned:
            self.state = 'displayed'
        else:
            self.state = 'hidden'
        self.scope.redraw()

    def menu_displayed(self, *args):
        self.state = 'displayed'
        self.pin.pos = self.width - self.pin.width, self.height

    def menu_incoming(self, *args):
        self.state = 'incoming'

    def menu_leaving(self, *args):
        self.pin.y = -self.pin.height
        self.state = 'leaving'

    def menu_hidden(self, *args):
        self.state = 'hidden'

       