import numpy as np

from kivy.lang import Builder
from kivy.uix.scatter import Scatter
from kivy.graphics.texture import Texture
from kivy.metrics import dp

Builder.load_string('''
<ImageScatter>:
    size: image.size
    size_hint: None, None 
    auto_bring_to_front: False
    do_rotation: True
    do_translation: True
    do_scale: True
    Image:
        id: image
''')

class ImageScatter(Scatter):

    def __init__(self, w=100, h=100, eyepiece=None, colorfmt='luminance', **kwargs):
        super(ImageScatter, self).__init__(**kwargs)
        self.eyepiece = eyepiece
        self.new_texture(w, h, colorfmt=colorfmt)
        self.w = w
        self.h = h
        self.colorfmt = colorfmt
        self.last_image = None

    def new_texture(self, w, h, colorfmt='luminance'):
        self.ids.image.size = w, h
        self.ids.image.texture = Texture.create(size=(h, w), colorfmt=colorfmt)
        self._set_center(self.eyepiece.origin)
        self.w, self.h = w, h
        self.colorfmt = colorfmt

    def display_blank(self):
        self.display(0 * np.ones((self.w, self.h)))

    def on_touch_down(self, touch):
        # check if touch is within eyepiece 
        if not (self.eyepiece.origin[0] - touch.pos[0])**2 + \
               (self.eyepiece.origin[1] - touch.pos[1])**2 < (self.eyepiece.radius - dp(60))**2:
            return False
        handled = super(ImageScatter, self).on_touch_down(touch)
        if self.collide_point(*touch.pos):
            if touch.is_double_tap:
                self.eyepiece.invert = not self.eyepiece.invert
                return True
        return handled

    def save(self, fn):
        self.ids.image.texture.save(fn)

    def display(self, x):

        w, h = x.shape
        if (w != self.w) or (h != self.h):
            self.new_texture(w, h)

        # only mono for now
        self.last_image = np.uint8(x*255)
        if self.colorfmt == 'luminance':
            self.ids.image.texture.blit_buffer(self.last_image.flatten(), 
                colorfmt='luminance', bufferfmt='ubyte')
        
        # else:
        #     green_chan = x.copy()
        #     blue_chan = x.copy()
        #     rgb = np.stack([x, green_chan, blue_chan], axis=-1)
        #     arr = np.uint8(np.asarray(rgb*255).flatten())
        #     self.ids.image.texture.blit_buffer(arr, colorfmt='rgb', bufferfmt='ubyte')