'''Base class for Jocular components to support load/save settings and
stubs for other methods 
'''

class Component:

    settables = []

    def load_settings(self, settings=None):
        if settings is None:
            return
        component = self.__class__.__name__
        if component in settings:
            subsettings = settings[component]
            for s in self.settables:
                if s in subsettings:
                    setattr(self, s, subsettings[s])

    def save_settings(self):
        component = self.__class__.__name__
        settings = {}
        for s in self.settables:
            settings[s] = getattr(self, s)
        return {component: settings}

    def on_close(self, *args):
        pass

    def post_init(self, *args):
        pass

    def redraw(self, *args):
        pass
