# class defining a combined knob/switch with presets
# in dire need of streamlining/simplification in next version!

import math
import random
from numpy import linspace as linspace

from kivy.uix.togglebutton import ToggleButton
from kivy.uix.behaviors import ToggleButtonBehavior
from kivy.lang import Builder
from kivy.uix.widget import Widget
from kivy.properties import (
    NumericProperty, ObjectProperty, StringProperty,
    BooleanProperty, BoundedNumericProperty,
    ListProperty, OptionProperty)
from kivy.uix.label import Label
from kivy.uix.floatlayout import FloatLayout
from kivy.clock import Clock

Builder.load_string('''

#---------------------------------------------------------------
# adapted/extended from Knob in Kivy Garden
#---------------------------------------------------------------

<SimpleKnob>
    size_hint: None, None

    canvas.before:
        Color:
            rgba: self.markeroff_color
        Ellipse:
            pos: self.pos
            size: self.size[0], self.size[1]
            angle_start: self.minangle + 360
            angle_end: self.maxangle + 360
        Color:
            rgba: self.marker_color
        Ellipse:
            pos: self.pos
            size: self.size[0], self.size[1]
            angle_start: self.minangle
            angle_end: self._angle if self._angle > self.minangle else self.minangle
        Color:
            rgba: 0, 0, 0, 1
        Ellipse:
            pos: self.pos[0] + (self.size[0] * (1 - self.track_proportion))/2, self.pos[1] + (self.size[1] * (1 - self.track_proportion)) / 2
            size: self.size[0] * self.track_proportion, self.size[1] * self.track_proportion
        PushMatrix
        Rotate:
            angle: 360 - self._angle
            origin: self.center
    canvas:
        PopMatrix

#-----------------------------------------------------
# A selector switch
#-----------------------------------------------------

<KnobSwitch>
    size_hint: None, None

<SwitchOption>
    size_hint: None, None
    canvas.before:
        Color:
            rgba: self.line_color
        Line:
            width: self.knob.line_width
            points: self.points 

    font_size: self.knob.label_font_size
    points: self.line_pos(self.knob.center, self.knob.radius, self.angle)
    size: self.texture_size
    pos: self.text_pos(self.knob.center, self.knob.radius, self.angle)
    background_color: self.knob.background_color
    line_color: self.color if self.knob.show_lines else (0, 0, 0, 0)
    color: self.knob.selected_color if self.state=='down' else self.knob.unselected_color
    on_state: self.knob.on_selection(self, self.text, self.state)
    state: 'down' if self.text == str(self.knob.value) else 'normal'

#-----------------------------------------------------
# A combined switch/knob with lots of functionality
#-----------------------------------------------------

<HyperKnob>
    knob: _knob
    switch: _switch
    toggle: _toggle
    value_label: _value_label
    size_hint: None, None

    SimpleKnob:
        id: _knob
        pos_hint: {'center_x': .5, 'center_y': .5}
        minvalue: root.min
        maxvalue: root.max
        show_marker: root.show_marker
        marker_color: root.marker_color
        markeroff_color: root.markeroff_color
        minangle: root.minangle
        maxangle: root.maxangle
        on_value: root.knob_value_changing()
        on_release: root.knob_value_changed()
        disabled: (not root.show_marker) or (_toggle.state == 'normal')
        on_disabled: root.inactive = self.disabled
        curve: root.curve

    KnobSwitch:
        id: _switch
        pos_hint: {'center_x': .5, 'center_y': .5}
        radius: root.radius * self.width
        line_end_radius: root.line_end
        line_start_radius: root.line_start
        labels: root.labels
        angles: root.angles
        selected_color: app.highlight_color
        unselected_color: _toggle.color
        on_value: root.switch_value_changed()

    Label:
        id: _value_label
        pos_hint: {'center_x': .5, 'center_y': .5}
        size: root.size
        text_size: self.size
        valign: 'middle'
        halign: 'center'
        font_size: root.value_font_size
        color: _toggle.color

    ToggleButton:
        id: _toggle
        pos_hint: {'center_x': .5, 'y': 0}
        #size: root.size
        size_hint: None, None
        size: root.width, root.title_font_size
        text_size: self.size
        valign: 'bottom'
        halign: 'center'
        font_size: root.title_font_size
        text: root.title
        color: root.text_color
        background_color: 0, 0, 0, 0
        state: 'normal' if root.inactive else 'down' 
        color: root.color if _toggle.state == 'down' else root.inactive_color

    ''')

class HyperKnob(FloatLayout):

    # final list of props here
    switch_radius = NumericProperty(.6)      # radius of disc part of switch as proportion of widget size
    knob_radius = NumericProperty(.5)        # likewise for knob part
    track_proportion = NumericProperty(.9)   # proportion of knob to use for track

    line_start = NumericProperty(.8)
    line_end = NumericProperty(1)

    radius = NumericProperty(.5)
    centre_disc_radius = NumericProperty(.8)

    inactive = BooleanProperty(False)
    show_knob = BooleanProperty(False)
    curve = NumericProperty(1.0)
    show_marker = BooleanProperty(False)
    show_discs = BooleanProperty(True)

    show_value = BooleanProperty(False)
    value_format = StringProperty('{:}')

    presets = ObjectProperty({})

    # value associated with closest preset
    hidden_presets = ObjectProperty({})

    presets_format = StringProperty('{:}')

    angles = ListProperty([])
    labels = ListProperty([])

    min = NumericProperty(0)
    max = NumericProperty(1)
   
    minangle = NumericProperty(0)
    maxangle = NumericProperty(360)
    # ending angle

    value = ObjectProperty(None)
    value_type = OptionProperty('float', options=['float', 'int', 'string'])
    #value = NumericProperty(0)

    knob_type = OptionProperty('numeric', options=['numeric', 'switch', 'numeric with presets', 'hidden presets'])

    title = StringProperty('')

    title_font_size = NumericProperty('20sp')
    value_font_size = NumericProperty('32sp')

    color = ListProperty([.6, .6, .6, 1]) # for text
    line_color = ListProperty([.2, .2, .2, 1])
    markeroff_color = ListProperty([.2, .2, .2, 1])
    marker_color = ListProperty([.4, .2, .2, 1])
    switch_color = ListProperty([.2, .2, .2, 1])
    #background_color = ListProperty([0, 0, 0, 1])
    background_color = ListProperty([0, 0, 0, 0])
    inactive_color = ListProperty([.3, .3, .3, 1])

    continuous_update = BooleanProperty(False)
    built = BooleanProperty(False)


    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        Clock.schedule_once(self._build_widget, 0)

    def prepare_value(self, v):
        if self.value_type == 'int':
            return round(v)
        else:
            return v

    def displayable_value(self, value):
        
        if type(value) is str:
            return value
        elif 'f' not in self.value_format:
            return self.value_format.format(int(value))
        else:
            return self.value_format.format(value)

    def _build_widget(self, *args):

        def v2a(v):
            fac = pow((v - self.min) / (self.max - self.min), 1./self.knob.curve)
            return fac * (self.maxangle - self.minangle) + self.minangle

        if self.built:
            return

        self.built = True

        if not self.title:
            self.toggle.height = 0

        mindim = min(self.width, self.height)
        self.switch.size = mindim * self.switch_radius, mindim * self.switch_radius
        self.knob.size = mindim * self.knob_radius, mindim * self.knob_radius
        self.knob.track_proportion = self.track_proportion
        self.toggle.state = 'down'

        # fill in defaults and construct only what needs showing

        if self.knob_type == 'numeric':
            self.show_marker = True
            self.knob.initialise_value(self.value)
            self.value_label.text = self.displayable_value(self.value)

        elif self.knob_type == 'hidden presets':
            self.show_marker = True
            self.min, self.max = -.1, len(self.presets) - 1
            self.show_value = True
            self.value_format = '{:}'
            self.knob_radius = 1
            self.knob.initialise_value(self.presets.index(self.value))
            self.value_label.text = self.value

        elif self.knob_type == 'switch':
            self.show_marker = False
            self.show_value = False
            self.markeroff_color = self.switch_color
            self.marker_color = self.switch_color
            self.line_color = self.switch_color

            if type(self.presets) == list: # generate equally-spaced angles
                labs = self.presets
                npts = len(labs)
                if (self.minangle == 0) and (self.maxangle == 360):
                    npts += 1  # avoid first and last points overlapping
                self.presets = {l: a for l, a in zip(labs, list(linspace(self.minangle, self.maxangle, npts)))}
                
            self.labels = list(self.presets.keys())
            self.angles = list(self.presets.values())

        elif self.knob_type == 'numeric with presets':
            self.show_marker = True

            if type(self.presets) == list:      # numbers correspond to location on scale eg  exposure
                self.min = min(self.presets)
                self.max = max(self.presets)
                self.labels = [self.presets_format.format(v) for v in self.presets]
                self.angles = [v2a(v) for v in self.presets]
                self.presets = {l: v for l, v in zip(self.labels, self.presets)}
               
            else: # numbers correspond to angles on scale

                self.labels = list(self.presets.keys())
                self.angles = [v2a(v) for v in self.presets.values()]

        self.on_value()
        self.knob_value_changed()


    '''
    Called when value released on Knob component. Actions:

    numeric:                update value (with maybe some type conversion); update value label
    hidden presets:         set preset closest to value; update value label
    switch:                 nothing
    numeric with presets:   update value; update value label

    '''

    def knob_value_changed(self, *args):

        if not self.built:
            return

        if self.knob_type == 'numeric':
            self.value = self.prepare_value(self.knob.value)

        elif self.knob_type == 'hidden presets':
            # find closest
            cat_val = min(len(self.presets)-1, max(0, int(self.knob.value)))
            self.value = self.presets[cat_val]

        elif self.knob_type == 'numeric with presets':
            self.value = self.prepare_value(self.knob.value)
            if self.value not in self.presets.values():
                self.switch.value = ''  # dehighlight selection

    # update value label when changing
    def knob_value_changing(self, *args):

        if not self.built:
            return

        if self.show_value:
            if self.knob_type in ['numeric', 'numeric with presets']:
                self.value_label.text = self.displayable_value(self.knob.value)
            elif self.knob_type == 'hidden presets':
                cat_val = min(len(self.presets)-1, max(0, int(self.knob.value)))
                self.value_label.text = self.presets[cat_val]

        if self.continuous_update:
            self.knob_value_changed()

    '''
    Called when value changes on switch component. Actions:

    numeric:                nothing
    hidden presets:         nothing
    switch:                 update value
    numeric with presets:   update value; update knob value

    '''

    def switch_value_changed(self, *args):

        if not self.built:
            return

        if not self.switch.value:
            return

        if self.knob_type == 'switch':
            self.value = self.switch.value

        elif self.knob_type == 'numeric with presets':

            self.knob.value = self.presets[self.switch.value]
            self.value = self.prepare_value(self.knob.value)


    def on_value(self, *args):
        # force initialisation if required

        if not self.built:
            return

        if self.knob_type == 'numeric':
            self.knob.value = self.value

        elif self.knob_type == 'hidden presets':
            # look up value associated with this preset
            if self.value in self.presets:
                self.knob.value = self.presets.index(self.value)

        elif self.knob_type == 'switch':
            self.switch.value = self.value

        elif self.knob_type == 'numeric with presets':
            self.knob.value = self.value
            #self.switch.value = self.value

            if self.knob.value in self.presets.values():
                self.switch.value = [k for k, v in self.presets.items() if v == self.knob.value][0]
            else:
                self.switch.value = ''
 
#--------------------------------------------------
# subsidiary classes

class SimpleKnob(Widget):

    minvalue = NumericProperty(0)
    maxvalue = NumericProperty(100)
    minangle = NumericProperty(0)     # for angles, negative implies LH quadrants
    maxangle = NumericProperty(360)
    value = NumericProperty(0)
    curve = BoundedNumericProperty(1, min=1)
    track_proportion  = BoundedNumericProperty(0.9, max=1.0, min=0.1)
    show_marker = BooleanProperty(False)

    marker_color = ListProperty([.5, .5, .5, 1])
    markeroff_color = ListProperty([0, 0, 0, 0])

    release = BooleanProperty(False)
    disabled = BooleanProperty(False)

    _angle          = NumericProperty(0)            # Internal angle calculated from value.
    _angle_step     = NumericProperty(0)            # Internal angle_step calculated from step.

    def __init__(self, *args, **kwargs):

        super().__init__(*args, **kwargs)
        self.bind(show_marker = self._show_marker)
        self.bind(value = self._value)
        self.selected = False

    def initialise_value(self, value):
        fac = pow((value - self.minvalue) / (self.maxvalue - self.minvalue), 1./self.curve)
        self._angle = fac * (self.maxangle - self.minangle) + self.minangle
        self.on_knob(value)

    def _value(self, instance, value):
        if self.disabled:
            return
        if value <= self.minvalue:
            fac = 0
        else:
            fac = pow((value - self.minvalue) / (self.maxvalue - self.minvalue), 1./self.curve)
        self._angle = fac * (self.maxangle - self.minangle) + self.minangle
        self.on_knob(value)

    def _show_marker(self, instance, flag):
        # "show/hide" marker.
        if flag:
            self.marker_color[3] = 1
            self.markeroff_color[3] = 1
        else:
            self.marker_color[3] = 0
            self.markeroff_color[3] = 0

    def on_touch_down(self, touch):
        if self.collide_point(*touch.pos):
            if self.show_marker:
                self.update_angle(touch)
                self.selected = True
                self.release = False
                return True
        return super().on_touch_down(touch)

    def on_touch_move(self, touch):
        if self.selected:
            if self.disabled:
                return
            self.update_angle(touch)

    def on_touch_up(self, touch):
        self.selected = False
        self.release = True

    def update_angle(self, touch):

        if self.disabled:
            return
        
        posx, posy          =   touch.pos
        cx, cy              =   self.center
        rx, ry              =   posx - cx, posy - cy

        if ry >= 0:                                 # Quadrants are clockwise.
            quadrant = 1 if rx >= 0 else 4
        else:
            quadrant = 3 if rx <= 0 else 2

        try:
            angle    = math.atan(rx / ry) * (180./math.pi)
            if quadrant == 2 or quadrant == 3:
                angle = 180 + angle
            elif quadrant == 4:
                angle = 360 + angle

        except: 
            angle = 90 if quadrant <= 2 else 270

        # don't update if angle is not in the allowed range
        if self.minangle >= 0:
            if angle < self.minangle:
                self.value = self.minvalue
                return
            elif angle > self.maxangle:
                self.value = self.maxvalue
                return
        elif angle < 180:
            if angle > self.maxangle:
                self.value = self.maxvalue
                return
        else:
            a = angle - 360
            if a < self.minangle:
                self.value = self.minvalue
                return

        self._angle_step    =   360 / (self.maxvalue - self.minvalue)
        self._angle         =   self._angle_step
        while self._angle < angle:
            self._angle     =   self._angle + self._angle_step

        if (self.minangle <= 0) and (self.maxangle >= 0):
            if angle > 180:
                angle -= 360

        relativeValue = (angle - self.minangle) / (self.maxangle - self.minangle)
        relativeValue = pow(relativeValue, self.curve)

        if relativeValue <= 0.:
            relativeValue = 0.
        elif relativeValue >= 1.:
            relativeValue = 1.
        else:
            self.value = (relativeValue * (self.maxvalue - self.minvalue)) + self.minvalue
               
    #TO OVERRIDE
    def on_knob(self, value):
        pass #Knob values listenerr

    def on_release(self, widgy, value):
        pass
        

class SwitchOption(ToggleButton):

    angle = NumericProperty(0.0)
    radius = NumericProperty(0.0)
    points = ListProperty([0, 0, 0, 0])
    line_color = ListProperty([0, 0, 0, 0])
    # disabled_color = ListProperty([.1, .1, .1, .5])
    knob = ObjectProperty(None)

    to_rad = 180 / math.pi

    def __init__(self, *args, **kwargs):

        super(SwitchOption, self).__init__(*args, **kwargs)
        # convert angle to 0-360 format with 0 at top, counting clockwise
        # first, convert to 0-360
        if self.angle < 0:
            self.angle += 360
        # now shift direction and by 90 degrees
        self.angle = 90 - self.angle
        if self.angle < 0:
            self.angle += 360
        
    def _do_press(self):
        if self.state == 'normal':
            ToggleButtonBehavior._do_press(self)

    def line_pos(self, origin, radius, angle):
        c = math.cos(angle/self.to_rad)
        s = math.sin(angle/self.to_rad)

        return origin[0] + radius*self.knob.line_start_radius*c, \
            origin[1] + radius*self.knob.line_start_radius*s, \
            origin[0] + radius*self.knob.line_end_radius*c, \
            origin[1] + radius*self.knob.line_end_radius*s \

    def text_pos(self, origin, radius, angle):
        # adjust for text size; find actual width/height from texture

        l = Label(text=self.text, text_size=(None, None))
        l.texture_update()
        w, h = l.texture_size
        margin = 5
        r = radius * self.knob.line_end_radius + margin
        cost, sint = math.cos(angle / self.to_rad), math.sin(angle / self.to_rad)
        return origin[0] + r * cost + (w * (cost + 1) / 2) - w, \
               origin[1] + r * sint + (h * (sint + 1) / 2) - h
     
class KnobSwitch(Label):

    angles = ListProperty([])
    labels = ListProperty([])
    value = StringProperty('')
    radius = NumericProperty(1)
    label_font_size = NumericProperty('16sp')
    font_size = NumericProperty('16sp')         # switch name
    line_width = NumericProperty(2)
    show_discs = BooleanProperty(True)
    show_lines = BooleanProperty(True)
    centre_disc_radius = NumericProperty(.75)

    line_start_radius = NumericProperty(.8)
    line_end_radius = NumericProperty(1.0)
    
    surround_disc_color = ListProperty([.2, .2, .2, 1])
    centre_disc_color = ListProperty([.13, .13, .13, 1])
    selected_color = ListProperty([1, 0, 0, .6])
    unselected_color = ListProperty([.5, .5, .5, .6])
    background_color = ListProperty([1, 0, 0, 0])
    name = StringProperty('')
    name_color = ListProperty([1, 0, 0, 1])

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.bind(show_discs=self._show_discs)
        self.bind(angles=self._build_widget, labels=self._build_widget) 
        self.built = False
        self.group = 'g' + str(random.randint(1, 9999)) # to distinguish from other instances
        self.options = []

    def _show_discs(self, instance, flag):
        if flag:
            self.surround_disc_color[3] = 1
            self.centre_disc_color[3] = 1
        else:
            self.surround_disc_color[3] = 0
            self.centre_disc_color[3] = 0

    def on_selection(self, widgy, selected, state):
        if selected == self.value:
            return
        if state == 'down':
            self.value = selected

    def _build_widget(self, *args):

        if self.built:
            return

        if self.labels and self.angles:
            self.built = True
            for label, ang in zip(self.labels, self.angles):
                if label == self.value:
                    state = 'down'
                else:
                    state = 'normal'
                o = SwitchOption(knob=self, angle=float(ang), state=state, text=str(label))
                self.options.append(o)
                self.add_widget(o)

