# Class to construct circular parts of the eyepiece, including orientation-sensitivity

from kivy.uix.widget import Widget
from kivy.vector import Vector

from kivy.lang import Builder
from jocular.utils.utils import angle360, angle_diff

Builder.load_string('''

<Ring>:
    canvas:
        Color:
            rgba: self.color
            rgba: self.grey, self.grey, self.grey, app.ring_transparency
        Line:
            circle: self.x, self.y, self.radius, self.start_angle, self.end_angle, self.nsegs
            width: self.thickness
''')

class Ring(Widget):

    def __init__(self, pos=[0, 0], radius=1, thickness=1, grey=.4, start_angle=0, end_angle=360, 
        nsegs=240, orientation_sensor=None):
        self.selected = False
        self.pos = pos
        self.radius = radius
        self.thickness = thickness
        self.grey = grey
        self.color = [grey, grey, grey, 1]
        self.start_angle = start_angle
        self.end_angle = end_angle
        self.nsegs = nsegs
        self.orientation_sensor = orientation_sensor
        super().__init__()

    def collide_point(self, touchpos):
        # check if we are in the ring zone (from radius - thickness to radius + thickness)
        dist = Vector(self.pos).distance(touchpos)
        return (dist > self.radius - self.thickness) and (dist < self.radius + self.thickness)

    def on_touch_down(self, touch):
        if self.orientation_sensor is not None:
            if self.collide_point(touch.pos):
                self.selected = True
                self.last_angle = angle_diff(self.pos, touch.pos)
                return True
        return super().on_touch_down(touch)

    def on_touch_move(self, touch):
        if not self.selected:
            return
        new_angle = angle_diff(self.pos, touch.pos)
        delta_angle = new_angle - self.last_angle
        self.orientation_sensor.orientation = angle360(self.orientation_sensor.orientation + delta_angle)
        self.last_angle = new_angle
        return True

    def on_touch_up(self, touch):
        self.selected = False

