'''General-purpose widgets for Jocular; also contains upper-right menu (case for
moving this out in next version)
'''

from kivy.lang import Builder
from kivy.uix.label import Label
from kivy.properties import BooleanProperty, StringProperty, NumericProperty
from kivy.uix.button import Button
from kivy.uix.togglebutton import ToggleButton
from kivy.uix.widget import Widget
from kivy.core.window import Window
from kivy.metrics import dp
from jocular.widgets.hyperknob import HyperKnob

Builder.load_string('''

#:import joc_icon jocular.utils.utils.joc_icon

<JLabel>:
    color: app.text_color
    markup: True
    halign: 'left'
    font_size: '10sp'

<JButton>:
    color: app.text_color
    disabled_color: app.lowlight_color
    text_size: self.size
    halign: 'left'
    size_hint: None, None
    background_color: 0, 0, 0, 0
    markup: True
    font_size: '16sp'

<JToggle>:
    markup: True
    text: '{:} {:}'.format(joc_icon(root.on_icon, fs=root.icon_size) if self.selected else joc_icon(root.off_icon, fs=root.icon_size), self.label)
    color: app.text_color if self.selected else app.lowlight_color
    text_size: self.size
    halign: 'left'
    size_hint: None, None
    size: dp(150), dp(28)

<JKnob>:
    pos_hint: {'center_x': .5, 'center_y': .5}
    size: dp(100), dp(100)
    minangle: -125
    maxangle: 125
    show_value: True
    value_font_size: '16sp'
    track_proportion: .85
    knob_radius: .9
    marker_color: app.knob_on_color
    markeroff_color: app.lowlight_color
    color: app.text_color
    inactive_color: app.lowlight_color
    title_font_size: '16sp'

<JSwitch>:
    pos_hint: {'center_x': .5, 'center_y': .5}
    knob_type: 'switch'
    size: dp(130), dp(130)
    switch_radius: .45
    knob_radius: .38
    track_proportion: .8
    marker_color: app.switch_color
    switch_color: app.switch_color
    markeroff_color: app.switch_color
    color: app.text_color

<MenuItem>:
    color: app.highlight_color if self.selected else app.text_color

<ConfigToggle>:
    size_hint: None, 1
    width: dp(28)
    text_size: self.size
    halign: 'center'
    valign: 'middle'
    markup: True
    text: joc_icon('config')
    background_normal: ''
    color: app.lowlight_color
    background_color: 0, 0, 0, 0
    border: (0,0,0,0)

''')

class JKnob(HyperKnob): pass
class JSwitch(HyperKnob): pass
class JButton(Button): pass
class JLabel(Label): pass
class ConfigToggle(ToggleButton): pass

class JToggle(Label):

    selected = BooleanProperty(False)
    disabled = BooleanProperty(True)
    label = StringProperty('')
    icon_size = NumericProperty('16sp')

    def __init__(self, selected=False, disabled=False, label='', on_icon='tick on', off_icon='tick off', **kwargs):
        self.selected = selected
        self.label = label
        self.disabled = disabled
        self.on_icon = on_icon
        self.off_icon = off_icon
        super().__init__(**kwargs)

    def on_touch_down(self, touch):
        if self.collide_point(*touch.pos):
            if not self.disabled:
                self.selected = not self.selected
                return True

#------------------------------
# upper right menu
#------------------------------

class MenuItem(JButton):

    selected = BooleanProperty(False)

    def __init__(self, fn=None, **kwargs):
        super().__init__(**kwargs)
        self.fn = fn
        Window.bind(mouse_pos=self.on_mouse_pos)

    def on_mouse_pos(self, *args):
        x, y = args[1]
        self.selected = self.collide_point(*self.to_widget(x, y))

class Menu(Widget):

    showing = BooleanProperty(False)

    def __init__(self, menu=None, text_width=150, **kwargs):

        super().__init__(**kwargs)
        self.button = Label(text='', color=(0, 0, 0, 0), **kwargs)
        self.add_widget(self.button)
        self.menu_items = {}
        self.text_width = text_width

        height = dp(28)
        x = Window.width
        y = Window.height - dp(5) #  - height

        for (label, fn) in menu:

            if label:
                y -= height
                self.menu_items[label] = MenuItem(
                    size=(self.text_width, height),
                    pos=(x, y),
                    font_size='16sp', 
                    halign='right',
                    fn=fn,
                    text=label)
                self.menu_items[label].bind(on_press=self.do_fn)
                self.add_widget(self.menu_items[label])

            else:
                y -= dp(14)

    def do_fn(self, instance):
        self.hide()
        instance.fn()

    def show(self, *args):
        if not self.showing:
            self.showing = True
            w = Window.width
            for m in self.menu_items.values():
                m.x = w - m.width - dp(5)

    def hide(self, *args):
        if self.showing:
            self.showing = False
            w = Window.width
            for m in self.menu_items.values():
                m.x = w
